package com.ibm.jps.util;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.NamedElement;

import com.ibm.xtools.modeler.ui.UMLModeler;

public class RSMResource {
	
	public static IMarker addMarker(IResource onRsrc, String description, int severity) {
        if (onRsrc == null) return null;
        try {
            IMarker marker = onRsrc.createMarker(IMarker.PROBLEM);
            if (marker.exists()) {
                marker.setAttribute(IMarker.MESSAGE, description);
                marker.setAttribute(IMarker.SEVERITY, severity);
                marker.setAttribute(IMarker.TRANSIENT, true);
                return marker;
            }
        } catch (CoreException e) {
            System.err.println("RSMModel.addMarker: unable to create marker.");
        }
        return null;
    }

	public static IConfigurationElement findConfigElement(IExtension inExt, String inConfigElementName) {
		IConfigurationElement ce = null;
		if (inExt != null) {
			for (IConfigurationElement ice : inExt.getConfigurationElements()) {
				if (ice.getAttribute("name").equals(inConfigElementName)) {
					ce = ice;
					break;
				}
			}
		}			
		return ce;
	}
	
    public static IExtension findExtension(String inExtensionPointName, String inExtensionId) {
		IExtensionRegistry eR = Platform.getExtensionRegistry();
		IExtensionPoint ep = eR.getExtensionPoint(inExtensionPointName);
		IExtension resExt = null;
		if (ep != null) {
			resExt = ep.getExtension(inExtensionId);
			if (resExt == null) {
				for (IExtension e : ep.getExtensions()) {
					if (e.getNamespaceIdentifier().equals(inExtensionId)) {
						resExt = e;
						break;
					}
				}
			}
		}		
		return resExt;
	}
    
    public static String getId(EObject inEObject) {
    	return inEObject.eResource().getURIFragment(inEObject);
    }

    public static IProject getProject(NamedElement elt) {
        Model model;
        if (elt instanceof Model)
            model = (Model) elt;
        else
            model = elt.getModel();
        Resource r = model.eResource();
        if (r != null) {
            IWorkspaceRoot wr = ResourcesPlugin.getWorkspace().getRoot();
            String uri = r.getURI().path().replaceAll("%20", " ");
            if (uri.startsWith("/resource/")) {
                String pName = uri.substring("/resource".length() + 1);
                int pos = pName.indexOf("/");
                if (pos > 0) {
                    pName = pName.substring(0, pos);
                    IProject prj = wr.getProject(pName);
                    return prj;
                }
            } else {
                // The following code handles the situation when a PathMap entry is defined: in this
                // case the uri is something like //PATHMAP_ENTRY/fileName.emx. Because I don't know
                // how to get to the Path Map entry, I came up with this awkward code: go through
                // all projects in the workspace, then locate the files 'fileName.emx'. If there are
                // duplicates, retrieve the corresponding model and look if it is equal to the input
                // model.
                int pos = uri.lastIndexOf("/");
                String fName = uri;
                if (pos >= 0) fName = fName.substring(pos + 1);
                List<IResource> resources = new ArrayList<IResource>();
                List<IProject> projects = new ArrayList<IProject>();
                for (int i = 0; i < wr.getProjects().length; i++) {
                    IProject prj = wr.getProjects()[i];
                    try {
                        for (int j = 0; j < prj.members().length; j++) {
                            IResource rsc = prj.members()[j];
                            if (rsc.getName().equals(fName)) {
                                projects.add(prj);
                                resources.add(rsc);
                            }
                        }
                    } catch (CoreException e) {
                        System.err
                                .println("RSMModel.getProject: Cannot retrieve the projects from the current workspace.");
                    }
                }
                if (!resources.isEmpty()) {
                    if (resources.size() == 1) {
                        return (IProject) projects.get(0);
                    } else {
                        for (int i = 0; i < resources.size(); i++) {
                            IResource rsc = (IResource) resources.get(i);
                            IFile f = (IFile) rsc;
                            Model m;
                            try {
                                m = UMLModeler.openModel(wr.getLocation().toOSString() + f.getFullPath().toOSString());
                                if (m.equals(model)) {
                                    return (IProject) projects.get(i);
                                }
                            } catch (IOException e) {
                                System.err.println("RSMModel.getProject: unable to open model for file "
                                        + wr.getLocation().toOSString() + f.getFullPath().toOSString());
                            }
                        }
                    }
                }
            }
        }
        return null;
    }

    public static IProject getProject(String projectName) {
        IWorkspaceRoot wr = ResourcesPlugin.getWorkspace().getRoot();
        for (int i = 0; i < wr.getProjects().length; i++) {
            IProject prj = wr.getProjects()[i];
            try {
                for (int j = 0; j < prj.members().length; j++) {
                    IResource rsc = prj.members()[j];
                    if (rsc.getName().equals(projectName)) {
                        return prj;
                    }
                }
            } catch (CoreException e) {
                System.err.println("RSMModel.getProject: Cannot retrieve the projects from the current workspace.");
            }
        }
        return null;
    }


}
